@@ -9,6 +9,7 @@ AP_NORTHEAST_2 = "ap-northeast-2"
 AP_NORTHEAST_3 = "ap-northeast-3"
 AP_SOUTHEAST_1 = "ap-southeast-1"
 AP_SOUTHEAST_2 = "ap-southeast-2"
+AP_SOUTHEAST_3 = "ap-southeast-3"
 AP_SOUTH_1 = "ap-south-1"
 CA_CENTRAL_1 = "ca-central-1"
 CN_NORTH_1 = "cn-north-1"
@@ -51,6 +52,8 @@ AP_NORTHEAST_2C = "ap-northeast-2c"
 AP_NORTHEAST_2D = "ap-northeast-2d"
 
 AP_NORTHEAST_3A = "ap-northeast-3a"
+AP_NORTHEAST_3B = "ap-northeast-3b"
+AP_NORTHEAST_3C = "ap-northeast-3c"
 
 AP_SOUTHEAST_1A = "ap-southeast-1a"
 AP_SOUTHEAST_1B = "ap-southeast-1b"
@@ -60,6 +63,10 @@ AP_SOUTHEAST_2A = "ap-southeast-2a"
 AP_SOUTHEAST_2B = "ap-southeast-2b"
 AP_SOUTHEAST_2C = "ap-southeast-2c"
 
+AP_SOUTHEAST_3A = "ap-southeast-3a"
+AP_SOUTHEAST_3B = "ap-southeast-3b"
+AP_SOUTHEAST_3C = "ap-southeast-3c"
+
 AP_SOUTH_1A = "ap-south-1a"
 AP_SOUTH_1B = "ap-south-1b"
 AP_SOUTH_1C = "ap-south-1c"
@@ -70,6 +77,7 @@ CA_CENTRAL_1D = "ca-central-1d"
 
 CN_NORTH_1A = "cn-north-1a"
 CN_NORTH_1B = "cn-north-1b"
+CN_NORTH_1C = "cn-north-1c"
 
 CN_NORTHWEST_1A = "cn-northwest-1a"
 CN_NORTHWEST_1B = "cn-northwest-1b"
